#include <Windows.h>
#include <stdio.h>
#include <tchar.h>

#include <mchd.h>

#define BOARD_INDEX 0
#define RUNNING_TIME_IN_SECONDS 10

#define PCM_FILE_NAME "C:\\test.pcm"


void MCHD_CALLING_CONVENTION AudioCallbackFunction(const MCHD_AUDIO_CALLBACK_INFO *info)
{
    printf("Callback for audio channel %p\n", info->channel);
    printf("  Frame data @ %p\n", info->frame.data);
    printf("        size = %i\n", info->frame.size);
    printf("        timestamp = %llu\n", info->frame.timestamp);

    if (info->context != NULL) {
        fwrite(info->frame.data, 1, info->frame.size, (FILE *)info->context);
    }

    printf("\n");
}


int _tmain(int argc, _TCHAR *argv[])
{
    MCHD_STATUS status;

    MCHD_BOARD board = MCHD_INVALID_HANDLE;    
    
    MCHD_AUDIO_CHANNEL audioChannel = MCHD_INVALID_HANDLE;

    MCHD_AUDIO_INPUT audioConnector = MCHD_AUDIO_INPUT_SDI;
    MCHD_AUDIO_SAMPLE_RATE audioSampleRate = MCHD_AUDIO_SAMPLE_RATE_32000;

    FILE *pcmFile = NULL;

    status = MchdInitialize();
    if (status != MCHD_STATUS_NO_ERROR) {
        goto cleanup;
    }

    status = MchdCreateBoard(BOARD_INDEX, &board);
    if (status != MCHD_STATUS_NO_ERROR) {
        goto cleanup;
    }

    status = MchdCreateAudioChannel(board, 
                                    audioConnector,
                                    audioSampleRate,
                                    &audioChannel);
    if (status != MCHD_STATUS_NO_ERROR) {
        MchdDeleteBoard(board);
        goto cleanup;
    }

    MchdDeleteBoard(board);
    board = MCHD_INVALID_HANDLE;

    fopen_s(&pcmFile, PCM_FILE_NAME, "wb+");

    status = MchdRegisterAudioChannelCallback(audioChannel, AudioCallbackFunction, pcmFile);
    if (status != MCHD_STATUS_NO_ERROR) {
        goto cleanup;
    }

    status = MchdStartAudioChannel(audioChannel);
    if (status != MCHD_STATUS_NO_ERROR) {
        goto cleanup;
    }

    Sleep(RUNNING_TIME_IN_SECONDS * 1000);

cleanup:
    if (audioChannel != MCHD_INVALID_HANDLE) {
        MchdStopAudioChannel(audioChannel);
        MchdDeleteAudioChannel(audioChannel);
        audioChannel = MCHD_INVALID_HANDLE;
    }

    if (pcmFile != NULL) {
        FILE *temp = pcmFile;
        pcmFile = NULL;
        fclose(temp);
    }

    MchdTerminate();

    if (status != MCHD_STATUS_NO_ERROR) {
        fprintf(stderr, "%s\n", MCHD_STATUS_DESCRIPTION(status));
    }

    printf("\nPress Enter to Continue...\n");
    getchar();

    return status;
}
