#include <Windows.h>
#include <stdio.h>
#include <tchar.h>

#include <mchd.h>

#define BOARD_INDEX 0
#define RUNNING_TIME_IN_SECONDS 10

#define H264_FILE_NAME "C:\\test.h264"


void MCHD_CALLING_CONVENTION VideoCallbackFunction(const MCHD_VIDEO_CALLBACK_INFO *info)
{
    printf("Callback for Video channel %p\n", info->channel);
    printf("  Surface data @ %p\n", info->surface.data);
    printf("          size = %i\n", info->surface.size);
    printf("          resolution = %ix%i\n", info->surface.resolution.width, info->surface.resolution.height);
    printf("          pitch = %i\n", info->surface.pitch);
    printf("          timestamp = %llu\n", info->surface.timestamp);
    if (info->surface.format == MCHD_VIDEO_COLOR_FORMAT_H264) {
        printf("          format = H264\n");
    } else {
        printf("          format = Unknown\n");
    }

    if (info->surface.format == MCHD_VIDEO_COLOR_FORMAT_H264) {
        if (info->context != NULL) {
            fwrite(info->surface.data[0], 1, info->surface.size[0], (FILE *)info->context);
        }
    }

    printf("\n");
}


int _tmain(int argc, _TCHAR *argv[])
{
    MCHD_STATUS status;

    MCHD_BOARD board = MCHD_INVALID_HANDLE;    
    
    MCHD_VIDEO_CHANNEL videoChannel = MCHD_INVALID_HANDLE;

    MCHD_VIDEO_INPUT videoConnector = MCHD_VIDEO_INPUT_SDI;
    MCHD_VIDEO_RESOLUTION videoResolution = MCHD_VIDEO_RESOLUTION_NATIVE;
    MCHD_VIDEO_COLOR_FORMAT videoFormat = MCHD_VIDEO_COLOR_FORMAT_H264;

    FILE *h264File = NULL;

    status = MchdInitialize();
    if (status != MCHD_STATUS_NO_ERROR) {
        goto cleanup;
    }

    status = MchdCreateBoard(BOARD_INDEX, &board);
    if (status != MCHD_STATUS_NO_ERROR) {
        goto cleanup;
    }

    status = MchdCreateVideoChannel(board, 
                                    videoConnector,
                                    videoResolution,
                                    videoFormat,
                                    &videoChannel);
    if (status != MCHD_STATUS_NO_ERROR) {
        MchdDeleteBoard(board);
        goto cleanup;
    }

    MCHD_VIDEO_CHANNEL_COMPRESSION_PARAMETERS params;
    params.profile = MCHD_VIDEO_COMPRESSION_PROFILE_MAIN;
    params.rateControlMode = MCHD_VIDEO_COMPRESSION_RATECONTROL_MODE_CBR;
    params.bitRate = 6000;
    params.gopSize = 50;

    status = MchdSetVideoChannelCompressionParameters(videoChannel, &params);
    if (status != MCHD_STATUS_NO_ERROR) {
        MchdDeleteBoard(board);
        goto cleanup;
    }

    MchdDeleteBoard(board);
    board = MCHD_INVALID_HANDLE;

    fopen_s(&h264File, H264_FILE_NAME, "wb+");

    status = MchdRegisterVideoChannelCallback(videoChannel, VideoCallbackFunction, h264File);
    if (status != MCHD_STATUS_NO_ERROR) {
        goto cleanup;
    }

    status = MchdStartVideoChannel(videoChannel);
    if (status != MCHD_STATUS_NO_ERROR) {
        goto cleanup;
    }

    Sleep(RUNNING_TIME_IN_SECONDS * 1000);

cleanup:
    if (videoChannel != MCHD_INVALID_HANDLE) {
        MchdStopVideoChannel(videoChannel);
        MchdDeleteVideoChannel(videoChannel);
        videoChannel = MCHD_INVALID_HANDLE;
    }

    if (h264File != NULL) {
        FILE *temp = h264File;
        h264File = NULL;
        fclose(temp);
    }

    MchdTerminate();

    if (status != MCHD_STATUS_NO_ERROR) {
        fprintf(stderr, "%s\n", MCHD_STATUS_DESCRIPTION(status));
    }

    printf("\nPress Enter to Continue...\n");
    getchar();

    return status;
}
