#include <Windows.h>
#include <stdio.h>
#include <tchar.h>

#include <mchd.h>
#include "display.h"

#define BOARD_INDEX 0
#define RUNNING_TIME_IN_SECONDS 10


HWND wnd;


void MCHD_CALLING_CONVENTION VideoCallbackFunction(const MCHD_VIDEO_CALLBACK_INFO *info)
{
    if (info->surface.format == MCHD_VIDEO_COLOR_FORMAT_YUYV) {
        printf("Callback for Video channel %p\n", info->channel);
        printf("  Surface data @ %p\n", info->surface.data[0]);
        printf("          size = %i\n", info->surface.size[0]);
        printf("          resolution = %ix%i\n", info->surface.resolution.width, info->surface.resolution.height);
        printf("          pitch = %i\n", info->surface.pitch[0]);
        printf("          timestamp = %llu\n", info->surface.timestamp);
        printf("          format = YUYV\n");
        DrawYUYVSurface(wnd, info->surface);
    } else if (info->surface.format == MCHD_VIDEO_COLOR_FORMAT_YV12) {
        printf("Callback for Video channel %p\n", info->channel);
        printf("  Surface data @ %p, %p, %p\n", info->surface.data[0], info->surface.data[1], info->surface.data[2]);
        printf("          sizes = %i, %i, %i\n", info->surface.size[0], info->surface.size[1], info->surface.size[2]);
        printf("          resolution = %ix%i\n", info->surface.resolution.width, info->surface.resolution.height);
        printf("          pitches = %i, %i, %i\n", info->surface.pitch[0], info->surface.pitch[1], info->surface.pitch[2]);
        printf("          timestamp = %llu\n", info->surface.timestamp);
        printf("          format = YV12\n");
        DrawYV12Surface(wnd, info->surface);
    } else {
        printf("          format = Unknown\n");
    }

    printf("\n");
}


int _tmain(int argc, _TCHAR *argv[])
{
    MCHD_STATUS status;

    MCHD_BOARD board = MCHD_INVALID_HANDLE;

    MCHD_VIDEO_CHANNEL videoChannel = MCHD_INVALID_HANDLE;

    MCHD_VIDEO_INPUT videoConnector = MCHD_VIDEO_INPUT_SDI;
    MCHD_VIDEO_RESOLUTION videoResolution = MCHD_VIDEO_RESOLUTION_NATIVE;
    MCHD_VIDEO_COLOR_FORMAT videoFormat = MCHD_VIDEO_COLOR_FORMAT_DEFAULT;

    wnd = NULL;

    status = MchdInitialize();
    if (status != MCHD_STATUS_NO_ERROR) {
        goto cleanup;
    }

    status = MchdCreateBoard(BOARD_INDEX, &board);
    if (status != MCHD_STATUS_NO_ERROR) {
        goto cleanup;
    }

    status = MchdCreateVideoChannel(board, 
                                    videoConnector,
                                    videoResolution,
                                    videoFormat,
                                    &videoChannel);
    if (status != MCHD_STATUS_NO_ERROR) {
        MchdDeleteBoard(board);
        goto cleanup;
    }

    MchdDeleteBoard(board);
    board = MCHD_INVALID_HANDLE;

    status = MchdRegisterVideoChannelCallback(videoChannel, VideoCallbackFunction, NULL);
    if (status != MCHD_STATUS_NO_ERROR) {
        goto cleanup;
    }

    if (wnd == NULL) {
        status = MchdGetVideoChannelResolution(videoChannel, &videoResolution);
        if (status != MCHD_STATUS_NO_ERROR) {
            goto cleanup;
        }

        wnd = CreateWin32Window(50, 50, videoResolution.width, videoResolution.height);
    }

    status = MchdStartVideoChannel(videoChannel);
    if (status != MCHD_STATUS_NO_ERROR) {
        goto cleanup;
    }

    Sleep(RUNNING_TIME_IN_SECONDS * 1000);

cleanup:
    if (videoChannel != MCHD_INVALID_HANDLE) {
        MchdStopVideoChannel(videoChannel);
        MchdDeleteVideoChannel(videoChannel);
        videoChannel = MCHD_INVALID_HANDLE;
    }

    MchdTerminate();

    if (wnd != NULL) {
        DestroyWindow(wnd);
    }

    if (status != MCHD_STATUS_NO_ERROR) {
        fprintf(stderr, "%s\n", MCHD_STATUS_DESCRIPTION(status));
    }

    printf("\nPress Enter to Continue...\n");
    getchar();

    return status;
}
